@@ -9,7 +9,6 @@
 #![feature(core_intrinsics)]
 #![feature(doc_cfg)]
 #![feature(exact_size_is_empty)]
-#![feature(forget_unsized)]
 #![feature(maybe_uninit_extra)]
 #![feature(maybe_uninit_ref)]
 #![feature(maybe_uninit_uninit_array)]
@@ -280,52 +279,6 @@ impl<T, const N: usize> StaticVec<T, N> {
     unsafe { &mut *ptr::slice_from_raw_parts_mut(self.as_mut_ptr(), self.length) }
   }
 
-  /// Returns a constant reference to the element of the StaticVec at `index`,
-  /// if `index` is within the range `0..length`. No checks are performed to
-  /// ensure that is the case, so this function is marked `unsafe` and should
-  /// be used with caution only when performance is absolutely paramount.
-  ///
-  /// Note that unlike [`slice::get_unchecked`](https://doc.rust-lang.org/nightly/std/primitive.slice.html#method.get_unchecked),
-  /// this method only supports accessing individual elements via `usize`; it cannot also produce
-  /// subslices. To unsafely get a subslice without a bounds check, use
-  /// `self.as_slice().get_unchecked(a..b)`.
-  ///
-  /// # Safety
-  ///
-  /// It is up to the caller to ensure that `index` is within the appropriate bounds.
-  #[inline(always)]
-  pub unsafe fn get_unchecked(&self, index: usize) -> &T {
-    debug_assert!(
-      index < self.length,
-      "Attempted to unsafely get at index {} when length is {}!",
-      index,
-      self.length
-    );
-    &*self.ptr_at_unchecked(index)
-  }
-
-  /// Returns a mutable reference to the element of the StaticVec at `index`,
-  /// if `index` is within the range `0..length`. No checks are performed to
-  /// ensure that is the case, so this function is marked `unsafe` and should
-  /// be used with caution only when performance is absolutely paramount.
-  ///
-  /// The same differences between this method and the slice method of the same name
-  /// apply as do for [`get_unchecked`](crate::StaticVec::get_unchecked).
-  ///
-  /// # Safety
-  ///
-  /// It is up to the caller to ensure that `index` is within the appropriate bounds.
-  #[inline(always)]
-  pub unsafe fn get_unchecked_mut(&mut self, index: usize) -> &mut T {
-    debug_assert!(
-      index < self.length,
-      "Attempted to unsafely get at index {} when length is {}!",
-      index,
-      self.length
-    );
-    &mut *self.mut_ptr_at_unchecked(index)
-  }
-
   /// Returns a constant pointer to the element of the StaticVec at `index` without doing any
   /// checking to ensure that `index` is within the range `0..length`. The return value of this
   /// function is equivalent to what would be returned from `as_ptr().add(index)`.
@@ -380,6 +333,52 @@ impl<T, const N: usize> StaticVec<T, N> {
     unsafe { self.mut_ptr_at_unchecked(index) }
   }
 
+  /// Returns a constant reference to the element of the StaticVec at `index`,
+  /// if `index` is within the range `0..length`. No checks are performed to
+  /// ensure that is the case, so this function is marked `unsafe` and should
+  /// be used with caution only when performance is absolutely paramount.
+  ///
+  /// Note that unlike [`slice::get_unchecked`](https://doc.rust-lang.org/nightly/std/primitive.slice.html#method.get_unchecked),
+  /// this method only supports accessing individual elements via `usize`; it cannot also produce
+  /// subslices. To unsafely get a subslice without a bounds check, use
+  /// `self.as_slice().get_unchecked(a..b)`.
+  ///
+  /// # Safety
+  ///
+  /// It is up to the caller to ensure that `index` is within the appropriate bounds.
+  #[inline(always)]
+  pub unsafe fn get_unchecked(&self, index: usize) -> &T {
+    debug_assert!(
+      index < self.length,
+      "Attempted to unsafely get at index {} when length is {}!",
+      index,
+      self.length
+    );
+    &*self.ptr_at_unchecked(index)
+  }
+
+  /// Returns a mutable reference to the element of the StaticVec at `index`,
+  /// if `index` is within the range `0..length`. No checks are performed to
+  /// ensure that is the case, so this function is marked `unsafe` and should
+  /// be used with caution only when performance is absolutely paramount.
+  ///
+  /// The same differences between this method and the slice method of the same name
+  /// apply as do for [`get_unchecked`](crate::StaticVec::get_unchecked).
+  ///
+  /// # Safety
+  ///
+  /// It is up to the caller to ensure that `index` is within the appropriate bounds.
+  #[inline(always)]
+  pub unsafe fn get_unchecked_mut(&mut self, index: usize) -> &mut T {
+    debug_assert!(
+      index < self.length,
+      "Attempted to unsafely get at index {} when length is {}!",
+      index,
+      self.length
+    );
+    &mut *self.mut_ptr_at_unchecked(index)
+  }
+
   /// Appends a value to the end of the StaticVec without asserting that
   /// its current length is less than `N`.
   ///
@@ -421,7 +420,9 @@ impl<T, const N: usize> StaticVec<T, N> {
   #[inline(always)]
   pub fn try_push(&mut self, value: T) -> Result<(), PushCapacityError<T, N>> {
     if self.length < N {
-      unsafe { self.push_unchecked(value) };
+      unsafe {
+        self.push_unchecked(value);
+      };
       Ok(())
     } else {
       Err(PushCapacityError::new(value))
@@ -432,9 +433,8 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// full; that is, if `self.len() == self.capacity()`.
   #[inline(always)]
   pub fn push(&mut self, value: T) {
-    self
-      .try_push(value)
-      .expect("Insufficient remaining capacity for push!")
+    assert!(self.length < N);
+    unsafe { self.push_unchecked(value) };
   }
 
   /// Removes the value at the last position of the StaticVec and returns it in `Some` if
@@ -562,6 +562,48 @@ impl<T, const N: usize> StaticVec<T, N> {
     }
   }
 
+  /// Functionally equivalent to [`insert`](crate::StaticVec::insert), except with multiple
+  /// items provided by an iterator as opposed to just one. This function will return immediately
+  /// if / when the StaticVec reaches maximum capacity, regardless of whether the iterator still has
+  /// more items to yield.
+  ///
+  /// For safety reasons, as StaticVec cannot increase in capacity, the
+  /// iterator is required to implement [`ExactSizeIterator`](core::iter::ExactSizeIterator)
+  /// rather than just [`Iterator`](core::iter::Iterator).
+  #[inline]
+  pub fn insert_many<I: IntoIterator<Item = T>>(&mut self, index: usize, iter: I)
+  where I::IntoIter: ExactSizeIterator<Item = T> {
+    assert!(self.length < N && index <= self.length);
+    let mut it = iter.into_iter();
+    if index == self.length {
+      return self.extend(it);
+    }
+    let iter_size = it.len();
+    assert!(index + iter_size >= index && (self.length - index) + iter_size < N);
+    unsafe {
+      let old_length = self.length;
+      let mut this = self.mut_ptr_at_unchecked(index);
+      this.copy_to(this.add(iter_size), old_length - index);
+      self.length = index;
+      let mut item_count = 0;
+      while item_count < N {
+        if let Some(element) = it.next() {
+          let mut current = this.add(item_count);
+          if item_count >= iter_size {
+            this = self.mut_ptr_at_unchecked(index);
+            current = this.add(item_count);
+            current.copy_to(current.offset(1), old_length - index);
+          }
+          current.write(element);
+          item_count += 1;
+        } else {
+          break;
+        }
+      }
+      self.length = old_length + item_count;
+    }
+  }
+
   /// Inserts `value` at `index` if the current length of the StaticVec is less than `N` and `index`
   /// is less than the length, or returns a [`CapacityError`](crate::errors::CapacityError)
   /// otherwise. Any values that exist in positions after `index` are shifted to the right.
@@ -804,17 +846,38 @@ impl<T, const N: usize> StaticVec<T, N> {
     res
   }
 
+  /// A version of [`concat`](crate::StaticVec::concat) for scenarios where `T` does not
+  /// derive [`Copy`](core::marker::Copy) but does implement [`Clone`](core::clone::Clone).
+  ///
+  /// Due to needing to call `clone()` through each individual element of `self` and `other`, this
+  /// function is less efficient than [`concat`](crate::StaticVec::concat), so
+  /// [`concat`](crate::StaticVec::concat) should be preferred whenever possible.
+  #[inline]
+  pub fn concat_clone<const N2: usize>(
+    &self,
+    other: &StaticVec<T, N2>,
+  ) -> StaticVec<T, { N + N2 }>
+  where
+    T: Clone,
+  {
+    let mut res = StaticVec::new();
+    for i in 0..self.length {
+      unsafe { res.push_unchecked(self.get_unchecked(i).clone()) };
+    }
+    for i in 0..other.length {
+      unsafe { res.push_unchecked(other.get_unchecked(i).clone()) };
+    }
+    res
+  }
+
   /// Returns a [`Vec`](alloc::vec::Vec) containing the contents of the StaticVec instance.
   /// The returned [`Vec`](alloc::vec::Vec) will initially have the same value for
   /// [`len`](alloc::vec::Vec::len) and [`capacity`](alloc::vec::Vec::capacity) as the source
-  /// StaticVec. Note that while using this function does *not* consume the source StaticVec in
-  /// the sense of rendering it completely inaccessible / unusable, it *does* empty it (that is,
-  /// it will have no contents and a length of 0 afterwards.)
+  /// StaticVec. Note that using this function consumes the source StaticVec.
   #[cfg(feature = "std")]
   #[doc(cfg(feature = "std"))]
-  #[allow(clippy::wrong_self_convention)]
   #[inline(always)]
-  pub fn into_vec(&mut self) -> Vec<T> {
+  pub fn into_vec(mut self) -> Vec<T> {
     let mut res = Vec::with_capacity(N);
     unsafe {
       self
