@@ -5,9 +5,11 @@ use core::cmp::{Eq, Ord, Ordering, PartialEq};
 use core::fmt::{self, Debug, Formatter};
 use core::hash::{Hash, Hasher};
 use core::iter::FromIterator;
-use core::ops::{Deref, DerefMut, Index, IndexMut};
+use core::ops::{
+  Deref, DerefMut, Index, IndexMut, Range, RangeFrom, RangeFull, RangeInclusive, RangeTo,
+  RangeToInclusive,
+};
 use core::ptr;
-use core::slice::SliceIndex;
 
 #[cfg(feature = "std")]
 use core::str;
@@ -244,24 +246,191 @@ impl<T: Hash, const N: usize> Hash for StaticVec<T, N> {
   }
 }
 
-impl<T, I: SliceIndex<[T]>, const N: usize> Index<I> for StaticVec<T, N> {
-  type Output = I::Output;
+impl<T, const N: usize> Index<usize> for StaticVec<T, N> {
+  type Output = T;
+  /// Asserts that `index` is less than the current length of the StaticVec,
+  /// and if so returns the value at that position as a constant reference.
   #[inline(always)]
-  fn index(&self, index: I) -> &Self::Output {
-    self.as_slice().index(index)
+  fn index(&self, index: usize) -> &Self::Output {
+    assert!(index < self.length);
+    unsafe { self.get_unchecked(index) }
   }
 }
 
-impl<T, I: SliceIndex<[T]>, const N: usize> IndexMut<I> for StaticVec<T, N> {
+impl<T, const N: usize> IndexMut<usize> for StaticVec<T, N> {
+  /// Asserts that `index` is less than the current length of the StaticVec,
+  /// and if so returns the value at that position as a mutable reference.
   #[inline(always)]
-  fn index_mut(&mut self, index: I) -> &mut Self::Output {
-    self.as_mut_slice().index_mut(index)
+  fn index_mut(&mut self, index: usize) -> &mut Self::Output {
+    assert!(index < self.length);
+    unsafe { self.get_unchecked_mut(index) }
+  }
+}
+
+impl<T, const N: usize> Index<Range<usize>> for StaticVec<T, N> {
+  type Output = [T];
+  /// Asserts that the lower bound of `index` is less than its upper bound,
+  /// and that its upper bound is less than or equal to the current length of the StaticVec,
+  /// and if so returns a constant reference to a slice of elements `index.start..index.end`.
+  #[inline(always)]
+  fn index(&self, index: Range<usize>) -> &Self::Output {
+    assert!(index.start < index.end && index.end <= self.length);
+    unsafe {
+      &*ptr::slice_from_raw_parts(self.ptr_at_unchecked(index.start), index.end - index.start)
+    }
+  }
+}
+
+impl<T, const N: usize> IndexMut<Range<usize>> for StaticVec<T, N> {
+  /// Asserts that the lower bound of `index` is less than its upper bound,
+  /// and that its upper bound is less than or equal to the current length of the StaticVec,
+  /// and if so returns a mutable reference to a slice of elements `index.start..index.end`.
+  #[inline(always)]
+  fn index_mut(&mut self, index: Range<usize>) -> &mut Self::Output {
+    assert!(index.start < index.end && index.end <= self.length);
+    unsafe {
+      &mut *ptr::slice_from_raw_parts_mut(
+        self.mut_ptr_at_unchecked(index.start),
+        index.end - index.start,
+      )
+    }
+  }
+}
+
+impl<T, const N: usize> Index<RangeFrom<usize>> for StaticVec<T, N> {
+  type Output = [T];
+  /// Asserts that the lower bound of `index` is less than or equal to the
+  /// current length of the StaticVec, and if so returns a constant reference
+  /// to a slice of elements `index.start()..self.length`.
+  #[inline(always)]
+  fn index(&self, index: RangeFrom<usize>) -> &Self::Output {
+    assert!(index.start <= self.length);
+    unsafe {
+      &*ptr::slice_from_raw_parts(
+        self.ptr_at_unchecked(index.start),
+        self.length - index.start,
+      )
+    }
+  }
+}
+
+impl<T, const N: usize> IndexMut<RangeFrom<usize>> for StaticVec<T, N> {
+  /// Asserts that the lower bound of `index` is less than or equal to the
+  /// current length of the StaticVec, and if so returns a mutable reference
+  /// to a slice of elements `index.start()..self.length`.
+  #[inline(always)]
+  fn index_mut(&mut self, index: RangeFrom<usize>) -> &mut Self::Output {
+    assert!(index.start <= self.length);
+    unsafe {
+      &mut *ptr::slice_from_raw_parts_mut(
+        self.mut_ptr_at_unchecked(index.start),
+        self.length - index.start,
+      )
+    }
+  }
+}
+
+impl<T, const N: usize> Index<RangeFull> for StaticVec<T, N> {
+  type Output = [T];
+  /// Returns a constant reference to a slice consisting of `0..self.length`
+  /// elements of the StaticVec, using [as_slice](crate::StaticVec::as_slice) internally.
+  #[inline(always)]
+  fn index(&self, _index: RangeFull) -> &Self::Output {
+    self.as_slice()
+  }
+}
+
+impl<T, const N: usize> IndexMut<RangeFull> for StaticVec<T, N> {
+  /// Returns a mutable reference to a slice consisting of `0..self.length`
+  /// elements of the StaticVec, using [as_mut_slice](crate::StaticVec::as_mut_slice) internally.
+  #[inline(always)]
+  fn index_mut(&mut self, _index: RangeFull) -> &mut Self::Output {
+    self.as_mut_slice()
+  }
+}
+
+impl<T, const N: usize> Index<RangeInclusive<usize>> for StaticVec<T, N> {
+  type Output = [T];
+  /// Asserts that the lower bound of `index` is less than or equal to its upper bound,
+  /// and that its upper bound is less than the current length of the StaticVec,
+  /// and if so returns a constant reference to a slice of elements `index.start()..=index.end()`.
+  #[inline(always)]
+  fn index(&self, index: RangeInclusive<usize>) -> &Self::Output {
+    assert!(index.start() <= index.end() && *index.end() < self.length);
+    unsafe {
+      &*ptr::slice_from_raw_parts(
+        self.ptr_at_unchecked(*index.start()),
+        (index.end() + 1) - index.start(),
+      )
+    }
+  }
+}
+
+impl<T, const N: usize> IndexMut<RangeInclusive<usize>> for StaticVec<T, N> {
+  /// Asserts that the lower bound of `index` is less than or equal to its upper bound,
+  /// and that its upper bound is less than the current length of the StaticVec,
+  /// and if so returns a mutable reference to a slice of elements `index.start()..=index.end()`.
+  #[inline(always)]
+  fn index_mut(&mut self, index: RangeInclusive<usize>) -> &mut Self::Output {
+    assert!(index.start() <= index.end() && *index.end() < self.length);
+    unsafe {
+      &mut *ptr::slice_from_raw_parts_mut(
+        self.mut_ptr_at_unchecked(*index.start()),
+        (index.end() + 1) - index.start(),
+      )
+    }
+  }
+}
+
+impl<T, const N: usize> Index<RangeTo<usize>> for StaticVec<T, N> {
+  type Output = [T];
+  /// Asserts that the upper bound of `index` is less than or equal to the
+  /// current length of the StaticVec, and if so returns a constant reference
+  /// to a slice of elements `0..index.end`.
+  #[inline(always)]
+  fn index(&self, index: RangeTo<usize>) -> &Self::Output {
+    assert!(index.end <= self.length);
+    unsafe { &*ptr::slice_from_raw_parts(self.as_ptr(), index.end) }
+  }
+}
+
+impl<T, const N: usize> IndexMut<RangeTo<usize>> for StaticVec<T, N> {
+  /// Asserts that the upper bound of `index` is less than or equal to the
+  /// current length of the StaticVec, and if so returns a constant reference
+  /// to a slice of elements `0..index.end`.
+  #[inline(always)]
+  fn index_mut(&mut self, index: RangeTo<usize>) -> &mut Self::Output {
+    assert!(index.end <= self.length);
+    unsafe { &mut *ptr::slice_from_raw_parts_mut(self.as_mut_ptr(), index.end) }
+  }
+}
+
+impl<T, const N: usize> Index<RangeToInclusive<usize>> for StaticVec<T, N> {
+  type Output = [T];
+  /// Asserts that the upper bound of `index` is less than the
+  /// current length of the StaticVec, and if so returns a constant reference
+  /// to a slice of elements `0..=index.end`.
+  #[inline(always)]
+  fn index(&self, index: RangeToInclusive<usize>) -> &Self::Output {
+    assert!(index.end < self.length);
+    unsafe { &*ptr::slice_from_raw_parts(self.as_ptr(), index.end + 1) }
+  }
+}
+
+impl<T, const N: usize> IndexMut<RangeToInclusive<usize>> for StaticVec<T, N> {
+  /// Asserts that the upper bound of `index` is less than the
+  /// current length of the StaticVec, and if so returns a constant reference
+  /// to a slice of elements `0..=index.end`.
+  #[inline(always)]
+  fn index_mut(&mut self, index: RangeToInclusive<usize>) -> &mut Self::Output {
+    assert!(index.end < self.length);
+    unsafe { &mut *ptr::slice_from_raw_parts_mut(self.as_mut_ptr(), index.end + 1) }
   }
 }
 
 #[cfg(feature = "std")]
 #[doc(cfg(feature = "std"))]
-impl<T, const N: usize> Into<Vec<T>> for &mut StaticVec<T, N> {
+impl<T, const N: usize> Into<Vec<T>> for StaticVec<T, N> {
   /// Functionally equivalent to [`into_vec`](crate::StaticVec::into_vec).
   #[inline(always)]
   fn into(self) -> Vec<T> {
