@@ -291,6 +291,43 @@ fn concat() {
   assert_eq!(v2, [4, 5, 6, 1, 2, 3]);
 }
 
+#[test]
+fn concat_clone() {
+  assert_eq!(
+    staticvec![Box::new("A, B")].concat_clone(&staticvec![
+      Box::new("C"),
+      Box::new("D"),
+      Box::new("E"),
+      Box::new("F")
+    ]),
+    [
+      Box::new("A, B"),
+      Box::new("C"),
+      Box::new("D"),
+      Box::new("E"),
+      Box::new("F")
+    ]
+  );
+  let v = StaticVec::<i32, 0>::from([]).concat_clone(&StaticVec::<i32, 0>::from([]));
+  assert_eq!(v, []);
+  let v2 = staticvec![Box::new(4), Box::new(5), Box::new(6)].concat_clone(&staticvec![
+    Box::new(1),
+    Box::new(2),
+    Box::new(3)
+  ]);
+  assert_eq!(
+    v2,
+    [
+      Box::new(4),
+      Box::new(5),
+      Box::new(6),
+      Box::new(1),
+      Box::new(2),
+      Box::new(3)
+    ]
+  );
+}
+
 #[test]
 fn dedup() {
   let mut vec = staticvec![1, 2, 2, 3, 2];
@@ -438,6 +475,9 @@ fn index() {
   assert_eq!(vec[1..4], [1, 2, 3]);
   assert_eq!(vec[1..=1], [1]);
   assert_eq!(vec[1..3], [1, 2]);
+  assert_eq!(vec[..3], [0, 1, 2]);
+  assert_eq!(vec[..=3], [0, 1, 2, 3]);
+  assert_eq!(vec[1..], [1, 2, 3, 4]);
   assert_eq!(vec[1..=3], [1, 2, 3]);
   assert_eq!(vec[..], [0, 1, 2, 3, 4]);
   // Because this block includes obviously-violated bounds checks, miri
@@ -463,6 +503,53 @@ fn insert() {
   assert_eq!(vec, [1, 4, 2, 3, 5]);
 }
 
+// The next few tests for `insert_many` are adapted from the SmallVec testsuite.
+
+#[test]
+fn insert_many() {
+  let mut v: StaticVec<u8, 8> = StaticVec::new();
+  for x in 0..4 {
+    v.push(x);
+  }
+  assert_eq!(v.len(), 4);
+  v.insert_many(1, [5, 6].iter().cloned());
+  assert_eq!(
+    &v.iter().map(|v| *v).collect::<StaticVec<_, 8>>(),
+    &[0, 5, 6, 1, 2, 3]
+  );
+  v.clear();
+  for x in 0..4 {
+    v.push(x);
+  }
+  assert_eq!(v.len(), 4);
+  v.insert_many(1, [5, 6].iter().cloned());
+  assert_eq!(
+    &v.iter().map(|v| *v).collect::<StaticVec<_, 8>>(),
+    &[0, 5, 6, 1, 2, 3]
+  );
+  v.clear();
+  for i in 0..6 {
+    v.push(i + 1);
+  }
+  v.insert_many(6, [1].iter().cloned());
+  assert_eq!(
+    &v.iter().map(|v| *v).collect::<StaticVec<_, 8>>(),
+    &[1, 2, 3, 4, 5, 6, 1]
+  );
+}
+
+#[test]
+#[should_panic]
+fn insert_many_asserts() {
+  let mut v: StaticVec<u8, 8> = StaticVec::new();
+  for i in 0..7 {
+    v.push(i + 1);
+  }
+  v.insert_many(0, [1, 2, 3, 4].iter().cloned());
+  let mut v2: StaticVec<u8, 0> = StaticVec::new();
+  v2.insert_many(27, [1, 2, 3, 4].iter().cloned());
+}
+
 #[test]
 fn is_empty() {
   let mut v = StaticVec::<i32, 1>::new();
@@ -537,7 +624,7 @@ fn into_iter() {
 #[cfg(feature = "std")]
 #[test]
 fn into_vec() {
-  let mut v = staticvec![
+  let v = staticvec![
     Box::new(Struct { s: "AAA" }),
     Box::new(Struct { s: "BBB" }),
     Box::new(Struct { s: "CCC" })
@@ -545,13 +632,6 @@ fn into_vec() {
   let vv = v.into_vec();
   assert_eq!(vv.capacity(), 3);
   assert_eq!(vv.len(), 3);
-  assert_eq!(v.capacity(), 3);
-  assert_eq!(v.len(), 0);
-  v.push(Box::new(Struct { s: "AAA" }));
-  v.push(Box::new(Struct { s: "BBB" }));
-  v.push(Box::new(Struct { s: "CCC" }));
-  assert_eq!(v.capacity(), 3);
-  assert_eq!(v.len(), 3);
 }
 
 #[test]
